import sys

from zope.app.event.objectevent import ObjectEvent
from zope.app.event.interfaces import IObjectEvent
from zope.interface import implements, Attribute
from zope.event import notify

from AccessControl import ClassSecurityInfo
from Products.CMFCore.WorkflowCore import ObjectDeleted
from Products.CMFCore.WorkflowCore import ObjectMoved
from Products.CMFCore.WorkflowCore import WorkflowException

# event interfaces

class IWorkflowActionEvent(IObjectEvent):
    
    """Base interface for events around workflow action invocation
    """
    
    workflow = Attribute("The workflow definition object")
    action = Attribute("The name of the action being invoked")
    
class IActionWillBeInvokedEvent(IWorkflowActionEvent):
    
    """Event fired immediately before a workflow action is invoked
    """
    
class IActionRaisedExceptionEvent(IWorkflowActionEvent):
    
    """Event fired when a workflow action raised an exception
    """
    
    exc = Attribute("The exception info for the exception raised")
    
class IActionSucceededEvent(IWorkflowActionEvent):
    
    """Event fired when a workflow action succeeded
    """
    
    result = Attribute("The result of the workflow action") 

# event classes

class WorkflowActionEvent(ObjectEvent):
    implements(IWorkflowActionEvent)
    
    def __init__(self, object, workflow, action):
        ObjectEvent.__init__(self, object)
        self.workflow = workflow
        self.action = action
    
class ActionWillBeInvokedEvent(WorkflowActionEvent):
    implements(IActionWillBeInvokedEvent)

            
class ActionRaisedExceptionEvent(WorkflowActionEvent):
    implements(IActionRaisedExceptionEvent)
    
    def __init__(self, object, workflow, action, exc):
        WorkflowActionEvent.__init__(self, object, workflow, action)
        self.exc = exc
    
class ActionSucceededEvent(WorkflowActionEvent):
    implements(IActionSucceededEvent)
    
    def __init__(self, object, workflow, action, result):
        WorkflowActionEvent.__init__(self, object, workflow, action)
        self.result = result

security = ClassSecurityInfo()

security.declarePrivate('_invokeWithNotification')
def _invokeWithNotification(self, wfs, ob, action, func, args, kw):

    """ Private utility method:  call 'func', and deal with exceptions
        indicating that the object has been deleted or moved.
    """
    reindex = 1
    for w in wfs:
        w.notifyBefore(ob, action)
        notify(ActionWillBeInvokedEvent(ob, w, action))
    try:
        res = func(*args, **kw)
    except ObjectDeleted, ex:
        res = ex.getResult()
        reindex = 0
    except ObjectMoved, ex:
        res = ex.getResult()
        ob = ex.getNewObject()
    except:
        exc = sys.exc_info()
        try:
            for w in wfs:
                w.notifyException(ob, action, exc)
                notify(ActionRaisedExceptionEvent(ob, w, action, exc))
            raise exc[0], exc[1], exc[2]
        finally:
            exc = None
    for w in wfs:
        w.notifySuccess(ob, action, res)
        notify(ActionSucceededEvent(ob, w, action, res))
    if reindex:
        self._reindexWorkflowVariables(ob)
    return res
