from zope.interface import implements
from zope.component import getMultiAdapter
from zope.component import getUtility

from Acquisition import aq_base, aq_inner, aq_parent
from Products.Five.browser import BrowserView

from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFPlone.interfaces import IBrowserDefault
from Products.CMFPlone.interfaces import INonStructuralFolder
from Products.CMFPlone.interfaces.NonStructuralFolder import INonStructuralFolder \
     as z2INonStructuralFolder

from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import utils

from eea.plone3compat.browser.interfaces import IContextState


class ContextState(BrowserView):
    """Information about the state of the current context
    """
    
    implements(IContextState)
    
    def view_url(self):
        """URL to use for viewing

        Files and Images get downloaded when they are directly
        called, instead of with /view appended.  We want to avoid that.
        """
        view_url = self.object_url()
        portal_properties = getToolByName(self.context, 'portal_properties', None)
        if portal_properties is not None:
            site_properties = getattr(portal_properties, 'site_properties', None)
            if site_properties is not None:
                use_view_action = site_properties.getProperty('typesUseViewActionInListings', ())
                if aq_base(self.context).portal_type in use_view_action:
                    view_url = view_url + '/view'
        return view_url

    def object_url(self):
        return aq_inner(self.context).absolute_url()
        
    def object_title(self):
        context = aq_inner(self.context)
        return utils.pretty_title_or_id(context, context)
        
    def workflow_state(self):
        tool = getToolByName(self.context, "portal_workflow")
        return tool.getInfoFor(aq_inner(self.context), 'review_state', None)
    
    def parent(self):
        return aq_parent(aq_inner(self.context))

    def is_folderish(self):
        return bool(getattr(aq_base(aq_inner(self.context)), 'isPrincipiaFolderish', False))
            
    def is_structural_folder(self):
        folderish = self.is_folderish()
        context = aq_inner(self.context)
        if not folderish:
            return False
        elif INonStructuralFolder.providedBy(context):
            return False
        elif z2INonStructuralFolder.isImplementedBy(context):
            # BBB: for z2 interface compat
            return False
        else:
            return folderish

    def is_default_page(self):
        context = aq_inner(self.context)
        container = aq_parent(context)
        if not container:
            return False
        view = getMultiAdapter((container, self.request), name='default_page')
        return view.isDefaultPage(context)
